<?php
/**
 * @package    Hikashop shipping plugin providing extra features
 * @author     https://www.brainforge.co.uk
 * @copyright  (C) 2023-2024 Jonathan Brain. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Extension;

use Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Helper\DateHelper;
use Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Plugin\PostcodePlugin;
use Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Traits\ConfigurationTrait;
use hikashopShippingPlugin;
use Joomla\CMS\Language\Text;
use Joomla\Event\DispatcherInterface;

\defined('_JEXEC') or die;

/**
 */
class BfExtraFeatures extends hikashopShippingPlugin {
	use ConfigurationTrait;

	protected $app;

	var $name = 'bfextrafeatures';

	/**
	 * Load the language file on instantiation.
	 *
	 * @var    boolean
	 * @since  3.1
	 */
	protected $autoloadLanguage = true;

	protected $weekdays = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday',];

	/**
	 * Constructor.
	 *
	 * @param   DispatcherInterface  $dispatcher  The dispatcher
	 * @param   array                $config      An optional associative array of configuration settings
	 *
	 * @since   4.3.0
	 */
	public function __construct(DispatcherInterface $dispatcher, array $config)
	{
		parent::__construct($dispatcher, $config);

		$this->multiple = true;
	}

	/*
	 */
	public function onShippingDisplay(&$order, &$dbrates, &$usable_rates, &$messages)
	{
		$local_usable_rates = [];

		$ret = parent::onShippingDisplay($order, $dbrates, $local_usable_rates, $local_messages);

		if ($ret === false || empty($local_usable_rates))
		{
			$messages['no_rates'] = Text::_('NO_SHIPPING_METHOD_FOUND');

			return false;
		}

		$shippingPostcodeMatchFound = false;

		$shippingPostcodeChecked = false;

		$shippingPostcodeAllowAny = false;

		$del = [];

		foreach ($local_usable_rates as $k => &$rate)
		{
			if (empty($rate->shipping_params->shipping_postcodeformat))
			{
				$shippingPostcodeAllowAny = true;
				continue;
			}

			$shippingPostcodeChecked = true;

			if (!empty($rate->shipping_params->shipping_ignorepostcodes) &&
				$shippingPostcodeMatchFound)
			{
				$del[] = $k;
				continue;
			}

			$hasMatch  = false;
			$eventData = array(&$hasMatch, $order,
				$rate->shipping_params->shipping_postcodeformat,
				$rate->shipping_params->shipping_validpostcodes,
				$this->params);

			$event = PostcodePlugin::dispatchEvent('onCheckPostcodeMatch', $eventData);

			$result = $event->getArgument('result') ?? [];

			if (!in_array(true, $result))
			{
				$del[] = $k;
				continue;
			}

			$shippingPostcodeMatchFound = true;
		}

		if ($shippingPostcodeChecked &&
			!$shippingPostcodeMatchFound &&
			!$shippingPostcodeAllowAny &&
			!empty($order->shipping_address))
		{
			$this->app->enqueueMessage(Text::sprintf('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_INVALIDPOSTCODE',
				trim(strtoupper($order->shipping_address->address_post_code ?? ''))), 'warning');
		}

		foreach ($del as $k) unset($local_usable_rates[$k]);

		if (empty($local_usable_rates))
		{
			$messages['no_postcoderate'] = Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_NOPOSTCODERATE');

			return false;
		}

		$dateObj = null;

		$latestordertimes = [];
		$del              = [];

		foreach ($local_usable_rates as $k => &$rate)
		{
			if (empty($rate->shipping_params->shipping_deliverydate)) continue;

			$mindaysbefore = ($rate->shipping_params->shipping_mindaysbefore ??= 0);
			$maxdaysafter  = ($rate->shipping_params->shipping_maxdaysafter ??= 0);

			if ($mindaysbefore == $maxdaysafter && $mindaysbefore == 0)
			{
				// Sameday delivery
				$dateObj ??= DateHelper::getDate();;

				if (!$this->canShipThisDate($dateObj, $rate))
				{
					$del[] = $k;
					continue;
				}

				$latestordertime = ($rate->shipping_params->shipping_latestordertime ?? '23:59');

				if ($dateObj->format('H:i', true) > $latestordertime)
				{
					$del[]              = $k;
					$latestordertimes[] = $latestordertime;
				}
			}
		}

		foreach ($del as $k) unset($local_usable_rates[$k]);

		if (empty($local_usable_rates))
		{
			rsort($latestordertimes);
			$messages['no_postcoderate'] =
				Text::sprintf('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_TOOLATESAMEDAY', $latestordertimes[0]);

			return false;
		}

		$itemDeliveryDate = self::getCartItemDeliveryDate($order);
		if ($itemDeliveryDate === false)
		{
			$local_usable_rates = [];
			$messages['itemnotondeliverydate'] = Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_ITEMNOTONDELIVERYDATE');

			return false;
		}

		if (!empty($itemDeliveryDate))
		{
			$dateObj = DateHelper::getDate($itemDeliveryDate);
			$del     = [];

			foreach ($local_usable_rates as $k => &$rate)
			{
				if (empty($rate->shipping_params->shipping_deliverydate)) continue;

				if ($this->canShipThisDate($dateObj, $rate)) continue;

				$del[] = $k;
			}

			foreach ($del as $k) unset($local_usable_rates[$k]);

			if (empty($local_usable_rates))
			{
				$messages['notforitemdeliverydate'] = Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_NOTFORITEMDELIVERYDATE');

				return false;
			}
		}

		foreach ($local_usable_rates as &$rate)
		{
			if (empty($rate->shipping_params->pricelist)) continue;

			$order_price = empty($rate->shipping_params->shipping_price_use_tax)
				? $order->full_total->prices[0]->price_value
				: $order->full_total->prices[0]->price_value_with_tax;

			$rate->shipping_price = $dbrates[$rate->shipping_id]->shipping_price;

			foreach ($rate->shipping_params->pricelist as $prices)
			{
				if (!is_array($prices)) continue;

				if (!empty($prices['minprice']) &&
					$prices['minprice'] > $order_price) continue;

				if (!empty($prices['maxprice']) &&
					$prices['maxprice'] < $order_price) continue;

				$rate->shipping_price += (float) $prices['price'];
				break;
			}
		}

		$usable_rates = $local_usable_rates;

		return true;
	}

	/*
	 */
	public function onAfterOrderConfirm(&$order, &$methods, $method_id)
	{
		return true;
	}

	/*
	 */
	protected function canShipThisDate($dateObj, $rate)
	{
		if (empty($dateObj)) return true;

		$weekdayParam = 'shipping_' . $this->weekdays[(int) $dateObj->format('w', true)];

		switch (intval($rate->shipping_params->$weekdayParam ?? 0))
		{
			case -1:
				return true;
			case 0:
				return false;
			default:
				// Too many orders on same day gets caught at onbeforeordercreate event.
				// Not checked here - might pass here then another order slips in before onbeforeordercreate event.
				return true;
		}
	}

	/*
	 */
	public static function getCartItemDeliveryDate($cart)
	{
		$itemHelperClass = '\Brainforgeuk\Component\HikashopBFItemDeliveryDates\Administrator\Helper\ItemHelper';
		if (!class_exists($itemHelperClass)) return null;

		return $itemHelperClass::getCartItemDeliveryDate($cart);
	}
}