<?php
/**
 * @package    Hikashop shipping plugin providing extra features
 * @author     https://www.brainforge.co.uk
 * @copyright  (C) 2023-2024 Jonathan Brain. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Plugin;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\Event\DispatcherInterface;
use Joomla\Event\Event;

\defined('_JEXEC') or die;

/**
 */
Class PostcodePlugin extends CMSPlugin
{
	protected $app;

	protected $autoloadLanguage = true;

	protected $postcodeType = [];

	protected $masterExtraFeatureParams = null;

	/*
	 */
	public function __construct(DispatcherInterface $dispatcher, array $config)
	{
		$classNameParts = explode('\\', get_class($this));

		$this->_type = strtolower($classNameParts[2]);  // Required for case-sensitive systems

		$this->_name = strtolower($classNameParts[3]);  // Required for case-sensitive systems

		parent::__construct($dispatcher, $config);
	}

	/*
	 */
	public function getShippingAddressPostcode($order)
	{
		$addressField = !empty($this->masterExtraFeatureParams) &&
						intval($this->masterExtraFeatureParams->get('usebillingpostcode'))
			? 'billing_address' : 'shipping_address';

		$shippingAddressPostcode = trim($order->$addressField->address_post_code ?? '');

		$shippingAddressPostcode = preg_replace('/\s+/', '', strtoupper($shippingAddressPostcode));

		if (empty($shippingAddressPostcode)) return '';

		return preg_match('/\A[A-Z0-9]+\z/', $shippingAddressPostcode) ? $shippingAddressPostcode : '';
	}

	/*
	 */
	public function onCheckPostcodeMatch(&$match, $order, $postcodeType, $validPostcodes, $extraFeatureParams)
	{
		if (!isset($this->postcodeType[$postcodeType])) return $match = false;

		$validPostcodes = $this->cleanValidPostcodes($validPostcodes);

		if (empty($validPostcodes)) return $match;

		$match = $validPostcodes[0] != '!';

		if (!$match)
		{
			// Must NOT match ANY of these postcodes
			$validPostcodes = trim(substr($validPostcodes, 1));

			if (empty($validPostcodes)) return !$match;
		}

		$this->masterExtraFeatureParams = $extraFeatureParams;
		$shippingAddressPostcode        = $this->getShippingAddressPostcode($order);

		if (empty($shippingAddressPostcode)) return $match = !$match;

		foreach(preg_split('/[\s,]+/', strtoupper($validPostcodes)) as $validPostcode)
		{
			if (strpos($validPostcode, '-') !== false &&
				strpos($validPostcode, '[') === false)
			{
				// Not a regular expression
				$postcodeRange = explode('-', $validPostcode, 2);

				// Should never happen, but just in case try to do something sensible
				if (strlen($postcodeRange[0]) == 0)
				{
					$postcodeRange[0] = '0';
				}

				// Should never happen, but just in case try to do something sensible
				if (count($postcodeRange) == 1 || strlen($postcodeRange[1]) == 0)
				{
					$postcodeRange[1] = str_pad('', strlen($postcodeRange[0]), '9');
				}

				$shippingAddressPostcodeStartsWith =
					substr($shippingAddressPostcode, 0, strlen($postcodeRange[0]));

				if (strcmp($shippingAddressPostcodeStartsWith, $postcodeRange[0]) >= 0 &&
					strcmp($shippingAddressPostcodeStartsWith, $postcodeRange[1]) <= 0)
				{
					return $match;
				}
			}
			else if (preg_match('/^' . $validPostcode . '/', $shippingAddressPostcode))
			{
				return $match;
			}
		}

		return $match = !$match;
	}

	/*
	 */
	public static function cleanValidPostcodes($validPostcodes)
	{
		$validPostcodeLines = [];

		foreach(explode("\n", $validPostcodes) as $validPostcodeLine)
		{
			$validPostcodeLine = trim($validPostcodeLine);

			if (empty($validPostcodeLine)) continue;

			if ($validPostcodeLine[0] == '#') continue;

			$validPostcodeLines[] = $validPostcodeLine;
		}

		return implode(' ', $validPostcodeLines);
	}

	/*
	 */
	public function onDisplayHikashopBfPostcodeFormatName(&$postcodeFormatName, $postcodeFormatCode)
	{
		if ($postcodeFormatName !== null) return;

		$postcodeFormatName = $this->postcodeType[$postcodeFormatCode] ?? null;
	}

	/*
	 */
	public function onHikashopBfPostcodeTypesLoad(&$postcodeTypes)
	{
		$postcodeTypes = array_merge($postcodeTypes ?? [], $this->postcodeType);
	}


	/*
	 */
	static public function dispatchEvent($eventName, &$eventData)
	{
		if (!PluginHelper::importPlugin('hikashopbfpostcodes')) return false;

		$event = new Event($eventName, $eventData);
		return Factory::getApplication()->getDispatcher()->dispatch($eventName, $event);
	}
}