<?php
/**
 * @package    Hikashop shipping plugin providing extra features
 * @author     https://www.brainforge.co.uk
 * @copyright  (C) 2023-2024 Jonathan Brain. All rights reserved.
 * @license    GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Traits;

use Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Helper\FieldHelper;
use Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Plugin\PostcodePlugin;
use Brainforgeuk\Plugin\HikashopShipping\BfExtraFeatures\Plugin\OrderDeliveryDatePlugin;
use hikaInput;
use Joomla\CMS\Factory;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Form\FormFactoryInterface;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;

\defined('_JEXEC') or die;

/**
 */
Trait ConfigurationTrait
{
	/*
	 */
	public function onShippingConfiguration(&$element)
	{
		if (!PluginHelper::isEnabled('hikashopshipping', 'bfextrafeatures'))
		{
			$app = Factory::getApplication();
			$app->enqueueMessage(Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_NOTENABLED'), 'warning');
			$app->redirect('index.php?option=com_hikashop&ctrl=plugins');

			return;
		}

		parent::onShippingConfiguration($element);

		Form::addFieldPath(dirname(__DIR__, 2) . '/forms');

		$form = Factory::getContainer()->get(FormFactoryInterface::class)->createForm('config', []);

		$xml = simplexml_load_file(dirname(__DIR__, 2) . '/forms/config.xml');

		$form->load($xml, false, null);

		?>
		<div id="bfextrafeatures-config">
			<?php
			$html   = [];
            
			echo HTMLHelper::_('uitab.startTabSet', 'bfShippingMethodTabs',
                ['active' => 'postcodestab', 'recall' => true, 'breakpoint' => 768]);

            foreach(['deliverydays', 'postcodes', 'prices', 'help'] as $fieldsetName)
            {
	            $html[] = HTMLHelper::_('uitab.addTab', 'bfShippingMethodTabs', $fieldsetName . 'tab',
		            Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_' . strtoupper($fieldsetName) . '_TAB'));

	            $html[] = $this->displayFieldset($element, $form->getFieldset($fieldsetName));

	            $html[] = HTMLHelper::_('uitab.endTab');
            }

			$html[] = HTMLHelper::_('uitab.endTabSet');

			echo implode('', $html);
			?>
		</div>

		<style>
            #bfextrafeatures-config .btn-check:checked + label.btn-outline-secondary,
            #bfextrafeatures-config .btn-check:checked + label.btn-outline-success {
                background-color: #457d54;
            }
            #bfextrafeatures-config .btn-check:checked + label.btn-outline-danger {
                background-color: #c52827;
            }
            #bfextrafeatures-config .shipping_extrafeatures_wide {
                width: 100%;
            }
            #bfextrafeatures-config .shipping_extrafeatures_overrides {
                width: 100%;
            }
            #bfextrafeatures-config .shipping_extrafeatures_notes {
                width: 100%;
                height: 10em;
            }
            #bfextrafeatures-config .shipping_extrafeatures-validpostcodes {
                width: 100%;
                height: 10em;
            }
            #bfextrafeatures-config .shipping_extrafeatures-note {
                width: 100%;
                text-align:center;
            }
            #pricestab > .control-group > .control-label {
                display: none;
            }
		</style>
		<?php
	}

	/*
	 */
	public function getShippingDefaultValues(&$element){
		$element->shipping_images = 'brainforge';
	}

	/*
	 */
	public function onShippingConfigurationSave(&$element)
	{
		foreach($element->shipping_params as $fieldName=>&$value)
		{
			switch($fieldName)
			{
				case 'shipping_latestordertime':
					if (!preg_match('/[0-2][0-9]:[0-5][0-9]/', $value) || $value >= '24:00')
					{
						$this->app->enqueueMessage(
							Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_LATESTORDERTIME_ERROR'), 'warning');

						$value = '11:59';
					}
					break;
				case 'shipping_monday_maxdeliveries':
				case 'shipping_tuesday_maxdeliveries':
				case 'shipping_wednesday_maxdeliveries':
				case 'shipping_thursday_maxdeliveries':
				case 'shipping_friday_maxdeliveries':
				case 'shipping_saturday_maxdeliveries':
				case 'shipping_sunday_maxdeliveries':
					$value = intval($value);
					if ($value < 0)
					{
						$value = 0 - $value;
					}
					break;
				case 'shipping_monday':
				case 'shipping_tuesday':
				case 'shipping_wednesday':
				case 'shipping_thursday':
				case 'shipping_friday':
				case 'shipping_saturday':
				case 'shipping_sunday':
					switch($value = intval($value))
					{
						case -1:
						case 1:
							break;
						default:
							$value = 0;
					}
					break;
				case 'shipping_extrafeatures_overrides':
					if (!$this->validateDeliverydateOverrides($value))
					{
						$this->app->enqueueMessage(
							Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_DELIVERYDATEOVERRIDES_ERROR'), 'warning');
					}
					break;
				case 'shipping_validpostcodes':
					$value = $this->uppercasePostcodes($value);
					break;
                case 'shipping_extrafeatures':
				case 'shipping_mindaysbefore':
                case 'shipping_maxdaysafter':
                    $value = intval($value);
                    break;
				default:
					break;
			}
		}
		unset($value);

		$pricelist = hikaInput::get()->get('shipping_params_pricelist', array(), 'array');

        foreach($pricelist as &$prices)
        {
            if (empty($prices)) continue;

            foreach($prices as &$price)
            {
                $price = str_replace(',', '.', $price);
	            if (empty($price))
	            {
		            $price = '';
		            continue;
	            }
	            $price = floatval($price);
            }
	        unset($price);
        }
		unset($prices);

		$element->shipping_params->pricelist = $pricelist;
	}

	/*
	 */
	protected function validateDeliverydateOverrides(&$value)
	{
		$value = trim($value);
		if (empty($value)) return true;

		$valid = true;

		$lines = preg_split('/[\n\r]+/', $value);

		foreach($lines as &$line)
		{
			$line = trim($line);

			if (empty($line)) continue;

			if ($line[0] == '#') continue;

			$parts = explode(':', $line);
			if (empty($parts[1]))
			{
				$parts[1] = 0;
			}
			else
			{
				$parts[1] = intval($parts[1]);
			}

			if (!preg_match('/^[.-[\]0-9]{8,}/U', $parts[0]))
			{
				$parts[0] = '# ERROR: ' . $parts[0];
				$valid = false;
			}

			$line = implode(':', $parts);
		}
		unset($line);

		$value = trim(implode("\n", $lines));

		return $valid;
	}

    /*
     */
	public function onAfterHikaPluginConfigurationListing($type, &$rows, &$listing_columns, &$obj)
    {
		if ($type != 'shipping') return;

	    foreach($rows as &$row)
	    {
		    if ($row->shipping_type != $this->name) continue;

		    $params = hikashop_unserialize($row->shipping_params);

		    if ($params->shipping_price_use_tax &&
			    (bccomp($params->shipping_min_price, 0, 5) || bccomp($params->shipping_max_price, 0, 5)))
		    {
			    $row->col_display_restriction .=
                    '<div class="bfextrafeatures_minmaxpricewithtax">' .
                        Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_MINMAXPRICEWITHTAX') .
                    '</div>';
		    }

		    if (!empty($params->shipping_postcodeformat))
		    {
			    $postcodeFormatName = null;

                $eventData = array( &$postcodeFormatName, $params->shipping_postcodeformat);
                PostcodePlugin::dispatchEvent('onDisplayHikashopBfPostcodeFormatName', $eventData);

			    $postcodeFormatName ??= $params->shipping_postcodeformat;

                $this->restrictionListingSpacer($row->col_display_restriction);

                $row->col_display_restriction .=
                    '<div class="bfextrafeatures_postcodetype">' .
                    Text::sprintf('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_POSTCODETYPE', Text::_($postcodeFormatName)) .
                    '</div>';

                $row->col_display_restriction .=
                    '<div class="bfextrafeatures_postcodes">' .
                    htmlspecialchars(PostcodePlugin::cleanValidPostcodes($params->shipping_validpostcodes)) .
                    '</div>';

                if (intval($params->shipping_ignorepostcodes))
                {
                    $row->col_display_restriction .=
                        '<div class="bfextrafeatures_postcodes">' .
                        Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_IGNORE_POSTCODES_LABEL') .
                        '</div>';
                }
		    }

		    if (OrderDeliveryDatePlugin::orderDeliveryDateFieldExists() &&
                intval($params->shipping_deliverydate))
		    {
			    $weekdays = [];
			    foreach([
				            'shipping_monday',  'shipping_tuesday',     'shipping_wednesday', 'shipping_thursday',
				            'shipping_friday',  'shipping_saturday',    'shipping_sunday',
			            ] as $shippingWeekday)
			    {
				    if (empty($params->$shippingWeekday)) continue;

				    $weekdays[] = Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_' .
					    strtoupper(substr($shippingWeekday, 9, 3)));
			    }

			    if (!empty($weekdays))
			    {
                    $this->restrictionListingSpacer($row->col_display_restriction);

				    $row->col_display_restriction .=
					    '<div class="bfextrafeatures_weekdays">' .
    					    implode(' ', $weekdays) .
					    '</div>';


				    $shipping_latestordertime = trim($params->shipping_latestordertime);
				    if (!empty($shipping_latestordertime))
				    {
					    $row->col_display_restriction .=
						    '<div class="bfextrafeatures_latestordertime">' .
                                Text::sprintf('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_LATESTORDERTIME',
                                    $shipping_latestordertime) .
						    '</div>';
				    }

				    $field = FieldHelper::getField('order');
                    $fieldOptions = $field->field_options['deliverydate_options'];

				    $deliverydate_overrides = [];
				    foreach(array_merge(
                            explode("\n", $params->shipping_extrafeatures_overrides ?? ''),
                            explode("\n", $fieldOptions['deliverydate_overrides'] ?? '')
                            )
                            as $deliverydate_override)
				    {
					    $deliverydate_override = trim($deliverydate_override);

					    if (empty($deliverydate_override) || $deliverydate_override[0] != '#') continue;

					    $deliverydate_override = htmlspecialchars(trim(substr($deliverydate_override, 1)));
                        if (!empty($deliverydate_override) &&
                            !in_array($deliverydate_override, $deliverydate_overrides))
                        {
	                        $deliverydate_overrides[] = $deliverydate_override;
                        }
				    }

				    if (!empty($deliverydate_overrides))
				    {
					    $row->col_display_restriction .=
						    '<div class="bfextrafeatures_deliverydateoverrides">' .
                                    implode('<br>', $deliverydate_overrides) .
						    '</div>';
				    }

				    switch ($params->shipping_extrafeatures_collectionlocker ?? '')
				    {
					    case 'M':
						    $lockerModeText = 'PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_COLLECTIONLOCKER_MANUAL';
						    break;
					    case 'A':
						    $lockerModeText = 'PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_COLLECTIONLOCKER_AUTOMATIC';
						    break;
                        default:
	                        $lockerModeText = null;
                            break;
				    }

                    if (!empty($lockerModeText))
                    {
	                    $row->col_display_restriction .=
		                    Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_COLLECTIONLOCKER_LABEL') .
                            ' : ' .
                            Text::_($lockerModeText);
                    }
			    }
		    }

		    if (!empty($params->pricelist))
		    {
			    $price_restrictions = [];

			    $currencyClass = hikashop_get('class.currency');

			    foreach($row->plugin_params->pricelist as $price)
			    {
				    if (empty($price)) continue;

                    if (empty($price['minprice']) && empty($price['maxprice'])) continue;

				    $price_restrictions[] =
					    ($price['minprice'] == '' ? 0 : $currencyClass->format($price['minprice'], $row->shipping_currency_id)) .
					    '->' .
					    ($price['maxprice'] == '' ? '&infin;' : $currencyClass->format($price['maxprice'], $row->shipping_currency_id)) .
					    '=' .
					    $currencyClass->format($price['price'], $row->shipping_currency_id) .
					    '<br>';
			    }

			    if (!empty($price_restrictions))
			    {
				    $this->restrictionListingSpacer($row->col_display_restriction);

				    $row->col_display_restriction .= Text::_('PLG_HIKASHOPSHIPPING_BFEXTRAFEATURES_VALUE_RESTRICTIONS') . '<br>';

				    $row->col_display_restriction .= implode('', $price_restrictions);
			    }
		    }
	    }
        unset($row);
	}

    /*
     */
    protected function restrictionListingSpacer(&$col_display_restriction)
    {
	    if (!empty($col_display_restriction))
	    {
		    $col_display_restriction .= '<hr style="margin:0;"/>';
	    }
    }

    /*
     */
    protected function uppercasePostcodes($postcodes)
    {
	    $values = [];
	    foreach(explode("\n", $postcodes) as $value)
	    {
		    $value = trim($value);
		    if (empty($value)) continue;

		    $values[] = ($value[0] == '#') ? $value : strtoupper($value);
	    }
	    return implode("\n", $values);
    }

    /*
     */
    protected function displayFieldset($element, $fieldset)
    {
	    $nameStart = 'data[shipping][shipping_params][';

        $html = [];

	    foreach ($fieldset as $field)
	    {
		    if (str_starts_with($field->fieldname, $nameStart))
		    {
			    $name = trim(substr($field->name, strlen($nameStart)), ']');

			    if (isset($element->shipping_params->$name))
			    {
				    $field->value = $element->shipping_params->$name;
			    }
		    }

		    switch($field->fieldname)
		    {
			    case 'data[shipping][shipping_params][shipping_latestordertime]':
				    if (!preg_match('/[0-2][0-9]:[0-5][0-9]/', $field->value) || $field->value >= '24:00')
				    {
					    $field->value = '11:59';
				    }
				    break;
			    case 'data[shipping][shipping_params][shipping_mindaysbefore]':
			    case 'data[shipping][shipping_params][shipping_maxdaysafter]':
			    case 'data[shipping][shipping_params][shipping_monday_maxdeliveries]':
			    case 'data[shipping][shipping_params][shipping_tuesday_maxdeliveries]':
			    case 'data[shipping][shipping_params][shipping_wednesday_maxdeliveries]':
			    case 'data[shipping][shipping_params][shipping_thursday_maxdeliveries]':
			    case 'data[shipping][shipping_params][shipping_friday_maxdeliveries]':
			    case 'data[shipping][shipping_params][shipping_saturday_maxdeliveries]':
			    case 'data[shipping][shipping_params][shipping_sunday_maxdeliveries]':
				    $field->value = intval($field->value);
				    if ($field->value < 0)
				    {
					    $field->value = 0 - $field->value;
				    }
				    break;
			    case 'data[shipping][shipping_params][shipping_monday]':
			    case 'data[shipping][shipping_params][shipping_tuesday]':
			    case 'data[shipping][shipping_params][shipping_wednesday]':
			    case 'data[shipping][shipping_params][shipping_thursday]':
			    case 'data[shipping][shipping_params][shipping_friday]':
			    case 'data[shipping][shipping_params][shipping_saturday]':
			    case 'data[shipping][shipping_params][shipping_sunday]':
				    switch($field->value = intval($field->value))
				    {
					    case -1:
					    case 1:
						    break;
					    default:
						    $field->value = 0;
				    }
				    break;
			    case 'data[shipping][shipping_params][shipping_extrafeatures_overrides]':
				    $value = $field->value;
				    $this->validateDeliverydateOverrides($value);
				    $field->value = $value;
				    break;
			    case 'data[shipping][shipping_params][shipping_validpostcodes]':
				    $field->value = $this->uppercasePostcodes($field->value);
				    break;
                case 'shipping_params_pricelist':
	                $field->value = $element->shipping_params->pricelist ?? [];
                    break;
			    default:
				    break;
		    }

		    $html[] = $field->renderField();
	    }

	    return implode('', $html);
    }
}